'use strict';

this.author  = 'Andrey Belov';
this.licence = 'CC-BY-NC-SA 3.0';

this.TFS = worldScripts['Target Filtering System'];

// Handle event "mode" for prime equipment
this.mode = function() {
    if ( player.ship.equipmentStatus('EQ_TARGET_FILTERING_SYSTEM') !== 'EQUIPMENT_OK' ) return;
    if ( ++this.TFS._mode > 5 ) this.TFS._mode = 0;
    this._showMsg( 'timer.tfs@mode', 'TFS mode: ' + ['hostile/outlaw','thargon','police','neutral','cargo','asteroids'][this.TFS._mode] );
};

this._showMsg = function(key, msg) {
    player.consoleMessage(msg, 1);
};

//Handle event "activated" for prime equipment
this.activated = function() {
    if ( player.ship.equipmentStatus('EQ_TARGET_FILTERING_SYSTEM') === 'EQUIPMENT_OK' ) {
        this._setNextTarget();
    }
};

this._setNextTarget = function() {
    var targets = [];

    if ( 0 === this.TFS._mode ) {
        targets = this._getHostileShips();

        if ( targets.length < 1 ) {
            targets = this._getOutlawShips();
        }
    }
    else if ( 1 === this.TFS._mode ) {
        targets = this._getThargoids();
    }
    else if ( 2 === this.TFS._mode ) {
        targets = this._getPolice();
    }
    else if ( 3 === this.TFS._mode ) {
        targets = this._getNeutralShips();
    }
    else if ( 4 === this.TFS._mode ) {
        targets = this._getCargoPods();
    }
    else if ( 5 === this.TFS._mode ) {
        targets = this._getAsteroids();
    }

    var len = targets.length - 1;

    if ( len < 0 ) {
        return;
    }

    this._sortByDistance(targets);

    var ptn = 0;
    if ( player.ship.target ) {
        ptn = TFS._getPreviousTargetNum();
        ptn = ptn >= len ? 0 : ptn + 1;
        TFS._setPreviousTargetNum(ptn);
    }
    var nextTarget = targets[ptn];

    if ( player.ship.target && player.ship.target === nextTarget ) {
        return;
    }

    player.ship.target = nextTarget;
};

this._getHostileShips = function() {
    if ( ! player.alertHostiles ) {
        return [];
    }
    return this._filterEntities(
        function(e) { return ( e.isShip && e.hasHostileTarget && e.target === player.ship ); }
    );
};

this._getOutlawShips = function() {
    if ( 1 > player.alertCondition ) {
        return [];
    }
    return this._filterEntities(
        function(e) { return ( e.scanClass === 'CLASS_NEUTRAL' && e.bounty !== 0 ); }
    );
};

this._getNeutralShips = function() {
    if ( 1 > player.alertCondition ) {
        return [];
    }
    return this._filterEntities(
        function(e) { return ( e.scanClass === 'CLASS_NEUTRAL' && e.bounty === 0 ); }
    );
};

this._getCargoPods = function() {
    return this._filterEntities(
        function(e) { return ( e.scanClass === 'CLASS_CARGO' ); }
    );
};

this._getThargoids = function() {
    return this._filterEntities(
        function(e) { return ( e.scanClass === 'CLASS_THARGOID' ); }
    );
};

this._getPolice = function() {
    return this._filterEntities(
        function(e) { return ( e.scanClass === 'CLASS_POLICE' ); }
    );
};

this._getAsteroids = function() {
    return this._filterEntities(
        function(e) { return ( e.scanClass === 'CLASS_ROCK' ); }
    );
};

this._filterEntities = function(filterFunc) {
    return system.filteredEntities(
        this,
        filterFunc,
        player.ship,
        player.ship.scannerRange
    );
};

this._sortByDistance = function(targets) {
    var cache = {};
    // sort
    targets.sort( function sortByDist(a,b) {
        // cache distance
        //var ai=a.ID,bi=b.ID; // work only with my patch with universalID getter :(
        var
        ai = a.entityPersonality,
        bi = b.entityPersonality; // use some "uniq" ident
        cache[ai] = cache[ai] || a.position.distanceTo(player.ship.position);
        cache[bi] = cache[bi] || b.position.distanceTo(player.ship.position);
        return ((cache[ai]<cache[bi])?-1:((cache[ai]>cache[bi])?1:0));
    } );
};
